describe PasswordRehasher do
	let(:plaintext_password) { "some_password" }
	let(:incorrect_plaintext_password) { "some_passwords" }

	let(:sha1_hashed_password) { "e4f85605cb02d2223d4f30c5ef5e6c38e662ae76" }
	let(:nested_hashed_password) { "nested hash: 400$8$39$bade652ec1b7cbb3$2e6efe19f8af6bb19a12ad9d62369fef50a12804f4be30bdc46c6626d3e07287" }
	let(:scrypt_hashed_password) { "400$8$38$76f69b1ead11cf5c$b0e509793a43e66d206d549cec5f039acf15b52fe965cd3b5d855408459c3ddb" }
	let(:invalid_hashed_password) { "400$8$38$76f60b1ead11cf5c$b0e509793a43e66d206d549cec5f039acf15b52fe965cd3b5d855408459c3ddb" }
	let(:salt) { "random salt" }

	describe '::VERSION' do
		subject { PasswordRehasher::VERSION }

		it { is_expected.to_not be_nil }
	end

	describe '.password_valid?' do
		subject { PasswordRehasher.password_valid?(plaintext_password, hashed_password, salt) }

		context 'with a SHA1 hash' do
			let(:hashed_password) { sha1_hashed_password }
			it { is_expected.to be_truthy }
		end

		context 'with a nested hash' do
			let(:hashed_password) { nested_hashed_password }
			it { is_expected.to be_truthy }
		end

		context 'with a scrypt hash' do
			let(:hashed_password) { scrypt_hashed_password }
			it { is_expected.to be_truthy }
		end

		context 'with an invalid scrypt hash' do
			let(:hashed_password) { invalid_hashed_password }
			it { is_expected.to be_falsey }
		end
	end

	describe '.rehash_needed?' do
		subject { PasswordRehasher.rehash_needed?(hashed_password) }

		context 'with a SHA1 hash' do
			let(:hashed_password) { sha1_hashed_password }
			it { is_expected.to be_truthy }
		end

		context 'with a nested hash' do
			let(:hashed_password) { nested_hashed_password }
			it { is_expected.to be_truthy }
		end

		context 'with a scrypt hash' do
			let(:hashed_password) { scrypt_hashed_password }
			it { is_expected.to be_falsey }
		end
	end

	describe '.nested_hash' do
		subject(:nested_hash) { PasswordRehasher.nested_hash(sha1) }

		context "with valid sha1 hash" do
			let(:sha1) { sha1_hashed_password }

			it "returns a valid nested hash" do
				expect(PasswordRehasher.password_valid?(plaintext_password, nested_hash, salt)).to be_truthy
			end
		end

		context "with an invalid sha1 hash" do
			let(:sha1) { 'haha lol nope' }

	 		it "returns invalid nested hash" do
				expect(PasswordRehasher.password_valid?(plaintext_password, nested_hash, salt)).to be_falsey
			end
		end
	end

	describe '.hash_password' do
		subject(:hashed_password) { PasswordRehasher.hash_password(plaintext_password) }

		it { is_expected.to be_a String }

		it "returns a password matching the plaintext password" do
			expect(PasswordRehasher.password_valid?(plaintext_password, hashed_password)).to be_truthy
		end

		it "returns a password that does not match a different plaintext password" do
			expect(PasswordRehasher.password_valid?(incorrect_plaintext_password, hashed_password)).to be_falsey
		end
	end

	describe '.validate_and_rehash?' do
		subject { PasswordRehasher.validate_and_rehash?(user, plaintext_password, hashed_password) }
		let(:user) { MockUser.new(hashed_password) }

		context 'with a SHA1 hash' do
			let(:hashed_password) { sha1_hashed_password }
			it { is_expected.to be_truthy }

			it "updates the stored crypted_password value" do
				expect { subject }.to change { user.crypted_password }.from(hashed_password)
			end
		end

		context 'with a nested hash' do
			let(:hashed_password) { nested_hashed_password }
			it { is_expected.to be_truthy }

			it "updates the stored crypted_password value" do
				expect { subject }.to change { user.crypted_password }.from(hashed_password)
			end
		end

		context 'with a scrypt hash' do
			let(:hashed_password) { scrypt_hashed_password }
			it { is_expected.to be_truthy }

			it "does not update the stored crypted_password value" do
				expect { subject }.to_not change { user.crypted_password }.from(hashed_password)
			end
		end

		context 'with an invalid scrypt hash' do
			let(:hashed_password) { invalid_hashed_password }
			it { is_expected.to be_falsey }

			it "does not update the stored crypted_password value" do
				expect { subject }.to_not change { user.crypted_password }.from(hashed_password)
			end
		end
	end
end

class MockUser
	attr_accessor :crypted_password

	def initialize(crypted_password)
		@crypted_password = crypted_password
	end

	def update_attribute(attribute, value)
		@crypted_password = value
		# attribute is always "crypted_password" for now
	end

	def salt
		"random salt"
	end
end